use chrono::{Datelike, Duration, Local, NaiveDate};
use rocket::{
    catch, catchers,
    fs::FileServer,
    get,
    request::FlashMessage,
    response::{Flash, Redirect},
    routes, Build, Rocket, State,
};
use rocket_dyn_templates::{tera::Context, Template};
use sqlx::SqlitePool;

use crate::model::{
    user::User,
    usertrip::{UserTrip, UserTripError},
    Day,
};

mod admin;
mod auth;
mod cox;

#[get("/")]
async fn index(db: &State<SqlitePool>, user: User, flash: Option<FlashMessage<'_>>) -> Template {
    let mut days = Vec::new();

    let mut show_next_n_days = 6;
    if user.is_cox {
        let end_of_year = NaiveDate::from_ymd_opt(Local::now().year(), 12, 31).unwrap();
        show_next_n_days = end_of_year
            .signed_duration_since(Local::now().date_naive())
            .num_days();
    }

    for i in 0..show_next_n_days {
        let date = (Local::now() + Duration::days(i)).date_naive();
        days.push(Day::new(db, date).await);
    }

    let mut context = Context::new();

    if let Some(msg) = flash {
        context.insert("flash", &msg.into_inner());
    }
    context.insert("loggedin_user", &user);
    context.insert("days", &days);
    Template::render("index", context.into_json())
}

#[get("/join/<trip_details_id>")]
async fn join(db: &State<SqlitePool>, trip_details_id: i64, user: User) -> Flash<Redirect> {
    match UserTrip::create(db, user.id, trip_details_id).await {
        Ok(_) => Flash::success(Redirect::to("/"), "Erfolgreich angemeldet!"),
        Err(UserTripError::EventAlreadyFull) => {
            Flash::error(Redirect::to("/"), "Event bereits ausgebucht!")
        }
        Err(UserTripError::AlreadyRegistered) => {
            Flash::error(Redirect::to("/"), "Du nimmst bereits teil!")
        }
        Err(UserTripError::AlreadyRegisteredAsCox) => {
            Flash::error(Redirect::to("/"), "Du hilfst bereits als Steuerperson aus!")
        }
    }
}

#[get("/remove/<trip_details_id>")]
async fn remove(db: &State<SqlitePool>, trip_details_id: i64, user: User) -> Flash<Redirect> {
    UserTrip::delete(db, user.id, trip_details_id).await;

    Flash::success(Redirect::to("/"), "Erfolgreich abgemeldet!")
}

#[catch(401)] //unauthorized
fn unauthorized_error() -> Redirect {
    Redirect::to("/auth")
}

pub fn start(db: SqlitePool) -> Rocket<Build> {
    rocket::build()
        .manage(db)
        .mount("/", routes![index, join, remove])
        .mount("/auth", auth::routes())
        .mount("/cox", cox::routes())
        .mount("/admin", admin::routes())
        .mount("/public", FileServer::from("static/"))
        .register("/", catchers![unauthorized_error])
        .attach(Template::fairing())
}

//#[cfg(test)]
//mod test {
//    use crate::testdb;
//
//    use super::start;
//    use rocket::http::Status;
//    use rocket::local::asynchronous::Client;
//    use rocket::uri;
//    use sqlx::SqlitePool;
//
//    #[sqlx::test]
//    fn test_not_logged_in() {
//        let pool = testdb!();
//
//        let client = Client::tracked(start(pool))
//            .await
//            .expect("valid rocket instance");
//        let response = client.get(uri!(super::index)).dispatch().await;
//
//        assert_eq!(response.status(), Status::SeeOther);
//        let location = response.headers().get("Location").next().unwrap();
//        assert_eq!(location, "/auth");
//    }
//}
